/*
    Common commands implementation.
    Copyright (C) 2011 2014 2015 2016  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#define __CMD_COMMON_C
#include "cmd_common.h"				// functions prototypes

#include <stdio.h>				// NULL
#include <string.h>
#include <unistd.h>
#include "text.h"				// text handling
#include "block.h"				// data block printing
#include "drivers/drivers.h"			// hardware drivers

/***************************************************************************/
/* define interni alla libreria */

#define BUF_SIZE	(8*1024)		// static buffer size

  // valori per il campo Mode
#define SHORT_LIST		0x00
#define LONG_LIST		0x01
#define FULL_LIST		0x03

/***************************************************************************/
/* macro interne alla libreria */

#define FieldCheck(F)	Supp="    "; if(F == NULL) Supp="not "

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* costanti interne alla libreria */

/***************************************************************************/
/* variabili interne alla libreria */

static const DrvInt_t	*Driver=NULL;	// driver attualmente selezionato
static unsigned char	BlockBuff[BUF_SIZE];	// data block buffer
static unsigned char	VerifyBuff[BUF_SIZE];	// data block buffer
static char		ReadBuf[1024];		// text read buffer

/***************************************************************************/
/* funzioni interne alla libreria */

static void DriverPrintAllInfo(int Mode);
  /* Stampa le info di tutti i driver supportati. */

static void DriverPrintInfo(const DrvInt_t *Chipset,int Mode);
  /* Stampa i comandi supportati dal chipset selezionato. */

static int DriverCheck(int(*Func)(void));
  /* Verifica che sia stato selezionato un driver, ritorna 0 in caso di
     successo, 1 se non c'e' un driver selezionato. */

/***************************************************************************/
/* funzioni */

static void DriverPrintAllInfo(int Mode)
{
  /* Stampa le info di tutti i driver supportati. */



  const DrvInt_t	*DrvPtr;		// puntatore al driver



  DrvPtr=ChipsetSearch(NULL);			// punta al primo della lista
  while(!IsEndOfList(DrvPtr))			// se la lista non e' finita
    DriverPrintInfo(DrvPtr++,Mode);		// stampa l'info block
  return;					// termina
}



static void DriverPrintInfo(const DrvInt_t *Chipset,int Mode)
{
  /* Stampa i comandi supportati dal chipset selezionato. */



  char	*Supp;					// pupported pointer
  
  
  
  if(Chipset==NULL)				// se il chipset non c'e'
    return;					// termina immediatamente
  if(Mode)
    putchar('\n');
  printf("%s\n",Chipset->DrvName);		// driver name
  if(Mode & 0x01)
  {
    printf("  file      : %s\n",Chipset->DrvFile);
    printf("  block size: %i\n",Chipset->BlockSize);
    printf("  blocks num: %i\n",Chipset->BlocksNum);
    printf("  total size: %i\n",Chipset->BlockSize * Chipset->BlocksNum);
    FieldCheck(Chipset->CmdSend);
    printf("  command   : %ssupported\n",Supp);
    FieldCheck(Chipset->CharWrite);
    printf("  write     : %ssupported\n",Supp);
    FieldCheck(Chipset->CharRead);
    printf("  read      : %ssupported\n",Supp);
    FieldCheck(Chipset->BlockWrite);
    printf("  bwrite    : %ssupported\n",Supp);
    FieldCheck(Chipset->BlockRead);
    printf("  bread     : %ssupported\n",Supp);
    FieldCheck(Chipset->BlockRead);
    printf("  bverify   : %ssupported\n",Supp);
  }
  if(Mode & 0x02)
    printf("  link:\n%s\n",Chipset->LinkMap);
  return;					// termina
}



static int DriverCheck(int(*Func)(void))
{
  /* Verifica che sia stato selezionato un driver, ritorna 0 in caso di
     successo, 1 se non c'e' un driver selezionato. */

  
  if(Driver == NULL)			// se c'e' un driver attivo
  {
    puts("No chipset selected, please select a chipset before.");
    return 1;				// termina con errore
  }
  if(Func == NULL)
  {
    puts("Function not avilable for this chipset.");
    return 1;				// termina con errore
  }
  return 0;				// termina con successo
}



int driverCmd(int argc,char **argv)
{
  /* Seleziona il driver attuale e avvia una nuova sessione. */



  const DrvInt_t	*DrvPtr;	// puntatore al driver



  if(argc==1)				// senza parametri
  {
    if(Driver==NULL)
    {
      puts("No driver seleted.");
      return 0;					// termina senza help
    }
    DriverPrintInfo(Driver,FULL_LIST);		// stampa tutto quanto
    return 0;					// termina con successo
  }

  // cerca il driver richiesto tra quelli disponibili
  DrvPtr=ChipsetSearch(argv[1]);	// punta al driver richiesto
  if(DrvPtr == NULL)			// se non lo trova
  {
    printf("Unknown driver: %s\n",argv[1]);
    return 1;				// termina e stampa l'help
  }
  if(!DrvPtr->BusInit((void*)DrvPtr,argc,argv))		// driver init
    Driver=DrvPtr;			// salva il driver attuale
  return 0;				// termina senza stampare l'help
}



int listCmd(int argc,char **argv)
{
  /* Elenca tutti i driver disponibili. */
  
  
  
  int	ListMode=SHORT_LIST;		// list mode



  puts("All available chipset drivers:");
  if(argc >= 2)				// se c'e' almeno un parametro
  {
    if(!strcmp(argv[1],"long"))
      ListMode=LONG_LIST;
    if(!strcmp(argv[1],"full"))
      ListMode=FULL_LIST;
  }
  DriverPrintAllInfo(ListMode);	// stampa la lista dei driver
  return 0;				// termina senza stampare l'help
}



int verboseCmd(int argc,char **argv)
{
  /* Imposta il livello di verbosita'. */
  
  
  
  if(argc==1)					// se non ha parametri
  {
    printf("Verbosity level: %i\n",Verbosity);
    return 0;					// termina con successo
  }
  if(argc==2)					// se ha 1 parametro
  {
    sscanf(argv[1],"%i",&Verbosity);		// imposta la verbosita'
    return 0;					// termina con successo
  }
  return 1;					// termina e stampa l'help
}



int commandCmd(int argc,char **argv)
{
  /* Invia una stringa di comandi al device attuale. */



  char			*CmdData;	// puntatore alla stringa comandi
  int			Len;
  int			Err;
  
  
  
  if(DriverCheck((int(*)(void))Driver->CmdSend))	// se non c'e' driver
    return 0;				// termina senza stampare l'help
  if(argc == 1)				// se manca il testo
    return 1;				// termina e stampa l'help
  CmdData=BuildTextStr(argc-1,&argv[1],&Len);	// crea la stringa comandi
  if((Err=Driver->CmdSend((unsigned char*)CmdData,Len)))
    printf("Chipset driver report I/O error (%i)\n",Err);
  return 0;				// termina con successo
}



int writeCmd(int argc,char **argv)
{
  /* Invia una stringa di testo al device attuale. */



  unsigned long int	Address=0;	// memory address
  int			Err;
  char			*Data;
  int			Len;



  if(DriverCheck((int(*)(void))Driver->CharWrite))	// se non c'e' driver
    return 0;				// termina senza stampare l'help
  if(argc < 3)				// se manca qualcosa
    return 1;				// termina e stampa l'help
  sscanf(argv[1],"%li",&Address);	// legge l'indirizzo
  Data=BuildTextStr(argc-2,&argv[2],&Len);	// build data string
  if((Err=Driver->CharWrite(Address,Data,Len)))	// write
    printf("Chipset driver report I/O error (%i)\n",Err);
  return 0;				// termina con successo
}



int readCmd(int argc,char **argv)
{
  /* Legge una stringa di testo dal device attuale. */
  
  
  
  unsigned long int	Address=0;		// memory address
  int			Len;			// num. byte da leggere
  int			Err;
  
  

  Len=Driver->BlockSize * Driver->BlocksNum;		// device total size
  if(DriverCheck((int(*)(void))Driver->CharRead))	// se non c'e' driver
    return 0;				// termina senza stampare l'help
  if(argc < 2)				// se manca l'indirizzo
    return 1;				// termina e stampa l'help
  sscanf(argv[1],"%li",&Address);	// legge l'indirizzo
  if(argc >= 3)				// se c'e' la lunghezza
    sscanf(argv[2],"%i",&Len);		// legge la lunghezza in byte
  if(Len > 1023)
    Len=1023;				// limita i disastri
  if((Err=Driver->CharRead(Address,ReadBuf,Len)))	// device read
  {
    printf("Chipset driver report I/O error (%i)\n",Err);
  }
  else
  {
    ReadBuf[Len]=0x00;			// termina la stringa
    puts(ReadBuf);			// stampa la stringa letta
  }
  return 0;				// termina con successo
}



int bwriteCmd(int argc,char **argv)
{
  /* Esegue la scrittura di un blocco dati sul device attuale. */



  FILE			*Src;		// file sorgente
  unsigned long int	Address=0;	// memory address
  int			Blocks=Driver->BlocksNum;	// numero di blocchi
  int			i;
  int			Err;



  if(DriverCheck((int(*)(void))Driver->BlockWrite))	// se non c'e' driver
    return 0;				// termina senza stampare l'help
  if(argc < 3)				// se manca qualcosa
    return 1;				// termina e stampa l'help
  sscanf(argv[1],"%li",&Address);	// legge l'indirizzo
  if(argc >= 4)				// se c'e' il numero di blocchi
    sscanf(argv[3],"%i",&Blocks);	// legge il numero di blocchi

  // apre il file sorgente
  if((Src=fopen(argv[2],"r"))==NULL)	// se ci sono problemi col file
  {
    perror("fopen");			// visualizza l'errore
    return 0;				// termina senza stampare l'help
  }

  // scrive i blocchi sul device
  while(Blocks)
  {
    for(i=0;i<Driver->BlockSize;i++)	// ripulisce il loading buffer
      BlockBuff[i]=0xff;
    if(!fread(BlockBuff,1,Driver->BlockSize,Src))	// se il file e' finito
      break;					// abbandona il ciclo
    printf("0x%08lx: ",Address);		// intesta la riga
    if((Err=Driver->BlockWrite(Address,BlockBuff,Driver->BlockSize)))	// wr
      printf("Chipset driver report I/O error (%i)\n",Err);
    else
      puts("ok");
    Address+=Driver->BlockSize;			// sposta l'indirizzo
    Blocks--;					// scala il contatore
  }
  fclose(Src);					// chiude il file sorgente
  return 0;
}



int breadCmd(int argc,char **argv)
{
  /* Esegue la lettura di un blocco dati dal device attuale. */
  
  
  
  FILE			*Dest=NULL;	// file destinazione
  unsigned long int	Address=0;	// memory address
  int			Blocks=Driver->BlocksNum;	// numero di blocchi
  int			Err;



  if(DriverCheck((int(*)(void))Driver->BlockRead))	// se non c'e' driver
    return 0;				// termina senza stampare l'help
  if(argc < 3)				// se manca qualcosa
    return 1;				// termina e stampa l'help
  sscanf(argv[1],"%li",&Address);	// legge l'indirizzo
  sscanf(argv[2],"%i",&Blocks);		// legge il numero di blocchi

  // apre il file destinazione
  if(argc >= 4)				// se c'e' il numero di blocchi
  {
    if((Dest=fopen(argv[3],"w"))==NULL)	// se ci sono problemi col file
    {
      perror("fopen");			// visualizza l'errore
      return 0;				// termina senza stampare l'help
    }
  }

  // legge i blocchi dal device
  while(Blocks)
  {
    if((Err=Driver->BlockRead(Address,BlockBuff,Driver->BlockSize)))	// rd
    {
      printf("Chipset driver report I/O error (%i)\n",Err);
    }
    else
    {
      if(Dest==NULL)				// se non c'e' file da salvare
      {
	PrintDataBlock(BlockBuff,Address,Driver->BlockSize);
      }
      else
      {
	fwrite(BlockBuff,1,Driver->BlockSize,Dest);	// salva il blocco
	printf("0x%08lx: saved\n",Address);
      }
    }
    Address+=Driver->BlockSize;			// sposta l'indirizzo
    Blocks--;					// scala il contatore
  }
  if(Dest!=NULL)
    fclose(Dest);			// chiude il file destinazione
  return 0;				// termina con successo
}



int bverifyCmd(int argc,char **argv)
{
  /* Esegue la verifica di un blocco dati dal device attuale. */
  
  
  
  FILE			*Src;		// file sorgente
  unsigned long int	Address=0;	// memory address
  int			Blocks=Driver->BlocksNum;	// numero di blocchi
  int			i;
  int			Err;



  if(DriverCheck((int(*)(void))Driver->BlockRead))	// se non c'e' driver
    return 0;				// termina senza stampare l'help
  if(argc < 4)				// se manca qualcosa
    return 1;				// termina e stampa l'help
  sscanf(argv[1],"%li",&Address);	// legge l'indirizzo
  sscanf(argv[2],"%i",&Blocks);		// legge il numero di blocchi

  // apre il file sorgente
  if((Src=fopen(argv[3],"r"))==NULL)	// se ci sono problemi col file
  {
    perror("fopen");			// visualizza l'errore
    return 0;				// termina senza stampare l'help
  }

  // verifica i blocchi dal device
  while(Blocks)
  {
    for(i=0;i<Driver->BlockSize;i++)	// ripulisce il loading buffer
      BlockBuff[i]=0xff;
    if(!fread(BlockBuff,1,Driver->BlockSize,Src))	// se il file e' finito
      break;					// abbandona il ciclo
    printf("0x%08lx: ",Address);		// intesta la riga
    if((Err=Driver->BlockRead(Address,VerifyBuff,Driver->BlockSize)))	// rd
    {
      printf("Chipset driver report I/O error (%i)\n",Err);
    }
    else
    {
      for(i=0;i<Driver->BlockSize;i++)
      {
        if(BlockBuff[i]!=VerifyBuff[i])
	{
	  puts("Failed");
	  return 0;				// termina il confronto
	}
      }
      puts("ok");
    }
    Address+=Driver->BlockSize;			// sposta l'indirizzo
    Blocks--;					// scala il contatore
  }
  fclose(Src);					// chiude il file sorgente
  return 0;					// termina con successo
}



int sleepCmd(int argc,char **argv)
{
  /* Blocca l'esecuzione per il numero indicato di secondi. */
    
    
  
  unsigned int	Time;			// durata della pausa



  if(argc < 2)				// se manca qualcosa
    return 0;				// termina senza stampare l'help
  sscanf(argv[1],"%ui",&Time);		// legge la duarat della pausa
  sleep(Time);				// pausa
  return 0;				// termina senza stampare l'help
}



int usleepCmd(int argc,char **argv)
{
  /* Blocca l'esecuzione per il numero indicato di microsecondi. */
  
  
  
  unsigned int	Time;			// durata della pausa



  if(argc < 2)				// se manca qualcosa
    return 0;				// termina senza stampare l'help
  sscanf(argv[1],"%ui",&Time);		// legge la duarat della pausa
  usleep(Time);				// pausa
  return 0;				// termina senza stampare l'help
}

/* $Source$ */

