/*
    I/O level commands implementation.
    Copyright (C) 2010 2011 2016  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#define __CMD_IO_C
//#include "cmd_io.h"				// functions prototypes

#include <stdio.h>
#include <string.h>
#define IO_ACCESS_ONLY	1			// include solo l'I/O level
#include "drivers/drivers.h"			// direct I/O control

/***************************************************************************/
/* costanti pubbliche */

const char	baudrateShortHelp[]=	"Set the hardware baudrate";
const char	baudrateHelp[]=		"<baudrate>\n\n"
		"Set the hardware auto update baudrate";
const char	pindirShortHelp[]=	"Set single pin direction";
const char	pindirHelp[]=		"[pin_number] [in|out]\n\n"
		"Set the direction of given pin, without parametrs show the\n"
		"I/O pins direction";
const char	pinrShortHelp[]=	"Read input pin status";
const char	pinrHelp[]=		"[pin_number]\n\n"
		"Read the instantaneus input level on specified pin, if any "
		"pin is specified\nall pins are read";
const char	pinwShortHelp[]=	"Write output pin status";
const char	pinwHelp[]=		"<pin_number> <hi|lo>\n\n"
		"Set the new level for an output pin";

/***************************************************************************/
/* define interni alla libreria */

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* costanti interne alla libreria */

/***************************************************************************/
/* variabili interne alla libreria */

/***************************************************************************/
/* funzioni interne alla libreria */

/***************************************************************************/
/* funzioni */

int baudrateCmd(int argc,char **argv)
{
  /* Imposta il baudrate di aggiornamento dell'hardware interface. */



  int	BaudRate;			// nuovo baudrate



  if(argc!=2)				// se gli argomenti sono sbagliati
    return 1;				// termina con fallimento
  sscanf(argv[1],"%i",&BaudRate);	// convete il nuovo baudrate
  IO_SetBaudRate(BaudRate);		// imposta il baudrate
  return 0;				// termina con successo
}



int dirmaskCmd(int argc,char **argv)
{
  /* Imposta/legge la direzione dei pin di I/O. */
  
  
  
  unsigned int	Mask;



  if(argc < 2)				// se manca il nuovo valore
  {
    printf("DirMask: 0x%02x\n",(unsigned int)IO_DirMaskRead());
    return 0;				// termina con successo
  }
  sscanf(argv[1],"%i",&Mask);		// legge il nuovo valore
  IO_DirMaskWrite((BusData_t)Mask);	// modifica le uscite
  return 0;				// termina con successo
}



int dataCmd(int argc,char **argv)
{
  /* Imposta/legge il valore delle linee di I/O. */
  
  
  
  unsigned int	Data;



  if(argc < 2)				// se manca il nuovo valore
  {
    printf("Data: 0x%02x\n",(unsigned int)IO_DataRead());
    return 0;				// termina con successo
  }
  sscanf(argv[1],"%i",&Data);		// legge il nuovo valore
  IO_DataWrite((BusData_t)Data);	// modifica le uscite
  return 0;				// termina con successo
}



int bitCmd(int argc,char **argv)
{
  /* Imposta/legge il valore di una singola linea di I/O. */
  
  
  
  unsigned int		Data;
  unsigned short int	Bit;



  if(argc < 2)				// se manca l'indica del bit
    return 1;				// termina con fallimento
  sscanf(argv[1],"%hi",&Bit);		// legge l'indice del bit scritto/letto
  if(argc < 3)				// se manca il nuovo valore
  {
    printf("Bit[%i]: %i\n",Bit,(unsigned int)IO_BitRead(Bit));
    return 0;				// termina con successo
  }
  sscanf(argv[2],"%i",&Data);		// legge il nuovo valore
  IO_BitWrite(Bit,(BusData_t)Data);	// modifica l'uscita
  return 0;				// termina con successo
}







int io_pinrCmd(int argc,char **argv)
{
  /* Legge lo stato di un pin di ingresso. */



  unsigned short int	Pin;			// numero pin letto
  
  
  
  switch(argc)
  {
    case 1:
      for(Pin=0;Pin<32;Pin++)
	if(IO_BitRead(Pin))
	  printf("pin %hi hi\n",Pin);
	else
	  printf("pin %hi lo\n",Pin);
    break;
    case 2:
      sscanf(argv[1],"%hi",&Pin);	// convete il pin
      if(IO_BitRead(Pin))
	puts("1");
      else
	puts("0");
    break;
    default:
      return 1;				// termina con fallimento
  }
  return 0;				// termina con successo
}



int io_pinwCmd(int argc,char **argv)
{
  /* Imposta lo stato di un pin di uscita. */
  
  
  
  unsigned short int	Pin;		// numero pin letto
  
  
  
  if(argc!=3)				// se gli argomenti sono sbagliati
    return 1;				// termina con fallimento
  sscanf(argv[1],"%hi",&Pin);		// convete il pin
  if(strcmp(argv[2],"hi")==0)
  {
    IO_BitSet(Pin);			// alza il pin
    return 0;				// termina con successo
  }
  if(strcmp(argv[2],"lo")==0)
  {
    IO_BitReset(Pin);			// abbassa il pin
    return 0;				// termina con successo
  }
  return 1;				// termina con fallimento
}

/* $Source$ */

