/*
    Command interpreter implementation.
    Copyright (C) 2011 2014 2015 2016  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#define __COMMAND_C
#include "command.h"				// functions prototypes

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include "parser.h"				// generazione di argv[], argc
#include "cmd_type.h"				// command data type
#include "cmd_io.h"				// basic I/O control
#include "cmd_spi.h"				// SPI bus control
#include "cmd_common.h"				// all devices commands

/***************************************************************************/
/* define interni alla libreria */

/***************************************************************************/
/* macro interne alla libreria */

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* costanti interne alla libreria */

/***************************************************************************/
/* variabili interne alla libreria */

static char	CmdName[32];			// nome comando + spazi bianchi

/***************************************************************************/
/* funzioni interne alla libreria */

static int testCmd(int argc,char **argv);
  /* Testa l'interprete dei comandi. */

static int helpCmd(int argc,char **argv);
  /* Visualizza l'help. */

static char *GetCmdName(const char *Name,int TotSpace);
  /* Dato il nome di un comando crea una stringa formata dal comando piu'
     un numero opportuno di spazi fino a arrivare a TotSpace.
     Ritorna un puntatore alla stringa. */

/***************************************************************************/
/* commands list */

static ShellCmd_t	CmdList[]={

ADD_SHELL_CMD(help,"Print command help or commands list",
"[command]\n\n"
"Print the command help, without args print the command list"),

ADD_SHELL_CMD(test,"Command interpreter test",
"<args>\n\n"
"Test the command interpreter"),

ADD_SHELL_CMD(sleep,"Suspends the program execution for given seconds",
"seconds\n\n"
"Suspends the program execution for given seconds"),

ADD_SHELL_CMD(usleep,"Suspends the program execution for given microseconds",
"seconds\n\n"
"Suspends the program execution for given microseconds"),

ADD_SHELL_CMD(verbose,"Set verbosity level",
"level\n\n"
"Set verbosity level"),

ADD_SHELL_CMD(driver,"Select the driver used and initialize the data bus",
"chipset <driver args>\n\n"
"Select the driver and reinitialize the bus, after this\n"
"command it is possible to use specific driver commands to drive the\n"
"connected device. Without parameters to obtain the selected driver"),

ADD_SHELL_CMD(list,"List all available drivers",
"list\n\n"
"List all available drivers"),



  // I/O level commands
ADD_SHELL_CMD(baudrate,"Set the hardware interface baudrate",
"newbaudrate\n\n"
"Set the hardware interface baudrate"),

ADD_SHELL_CMD(dirmask,"Set/show pin direction",
"pin_number <in|out>\n\n"
"Set pin direction if in|out is specified, otherwise print\n"
"current pin direction"),

ADD_SHELL_CMD(data,"Read/write data value",
"pin_number\n\n"
"Read and print current pin status"),

ADD_SHELL_CMD(bit,"Read/write single bit",
"pin_number [hi|lo]\n\n"
"Set pin status for output pins"),

/*
ADD_SHELL_CMD(io_pinr,"Read pin status",
"pin_number\n\n"
"Read and print current pin status"),

ADD_SHELL_CMD(io_pinw,"Set pin status",
"pin_number [hi|lo]\n\n"
"Set pin status for output pins"),
*/


  // I2C bus level commands



  // SPI bus level commands
ADD_SHELL_CMD(spi_mode,"Set the SPI bus control mode",
"mode word_bits\n\n"
"Set the SPI bus control mode and words length.\n"
"mode:\n"
"  0x01 CPOL HI\n"
"  0x02 SEC EDGE\n"
"  0x04 MSB FIRST\n"),

ADD_SHELL_CMD(spi_xchg,"Send/receive 1 word on SPI bus",
"data\n\n"
"Send/receive 1 word on SPI bus"),

ADD_SHELL_CMD(spi_write,"Send 1 word on SPI bus",
"data\n\n"
"Send 1 word on SPI bus"),



  // all devices commands
ADD_SHELL_CMD(command,"Send a command string to the active device",
"cmd1 <cmd2> <cmd3>...\n\n"
"Send a command bytes string to the active device"),

ADD_SHELL_CMD(write,"Write a text string to the active device",
"address text <more text>...\n\n"
"Write a text string to the active device at given address,\n"
"some driver (chipset) can ignore address parameter"),

ADD_SHELL_CMD(read,"Read a text string from the active device",
"address len\n\n"
"Read a text string from the active device at given address,\n"
"some driver (chipset) can ignore address parameter"),

ADD_SHELL_CMD(bwrite,"Write a data block to the active device",
"address filename <nblocks>\n\n"
"Write one or more data blocks to the active device at given address,\n"
"the number of written blocks is the minimum between the number of blocks\n"
"in the data file and the given nblocks parameter,\n"
"some driver (chipset) can ignore address parameter"),

ADD_SHELL_CMD(bread,"Read a data block from the active device",
"address nblocks <filename>\n\n"
"Read nblocks data blocks from the active device at given address,\n"
"if filename is given all data is saved in, otherwise read data\n"
"is printed to stdout"),

ADD_SHELL_CMD(bverify,"Read and check a data block from the active device",
"address nblocks filename\n\n"
"Read nblocks data blocks from the active device at given address,\n"
"and compare them vith data read from filename"),

{NULL,NULL,NULL,NULL}};					// end of list

/***************************************************************************/
/* funzioni */

static int testCmd(int argc,char **argv)
{
  /* Testa l'interprete dei comandi. */



  int	i;



  for(i=0;i<argc;i++)
    printf("argv[%i]=%s\n",i,argv[i]);
  return 0;				// termina con successo
}



static int helpCmd(int argc,char **argv)
{
  /* Visualizza l'help. */



  ShellCmd_t	*Entry;			// puntatore all'entry table



#if SHELL_CMD_SECTION
  Entry=&__shell_cmd_start;		// punta alla prima entry
#else
  Entry=CmdList;			// punta alla prima entry
#endif
  if(argc==1)
  {
    // stampa la lista dei comandi
    puts("\nCommands list:\n--------------");
#if SHELL_CMD_SECTION
    while(Entry!=&__shell_cmd_end)	// fino a fine tabella
#else
    while(Entry->ActFunc != NULL)	// fino al terminatore
#endif
    {
      printf("%s%s\n",GetCmdName(Entry->Cmd,14),Entry->ShortHelp);
      Entry++;
    }
    printf("\n");
  }
  else
  {
    // stampa l'help specifico del comando richiesto
#if SHELL_CMD_SECTION
    while(Entry!=&__shell_cmd_end)	// fino a fine tabella
#else
    while(Entry->ActFunc != NULL)	// fino al terminatore
#endif
    {
      if(strcmp(argv[1],Entry->Cmd)==0)
      {
        printf("Usage: %s %s.\n\n",Entry->Cmd,Entry->LongHelp);
	return 0;			// termina con successo
      }
      Entry++;				// entry successiva
    }
    printf("Unknown command: %s\n",argv[1]);
  }
  return 0;				// termina con successo
}



static char *GetCmdName(const char *Name,int TotSpace)
{
  /* Dato il nome di un comando crea una stringa formata dal comando piu'
     un numero opportuno di spazi fino a arrivare a TotSpace.
     Ritorna un puntatore alla stringa. */



  int	Len;				// lunghezza originale comando



  Len=strlen(Name);			// getta la lunghezza originale
  strcpy(CmdName,Name);			// copia il nome originale
  while(Len < TotSpace)
    CmdName[Len++]=' ';			// aggiunge uno spazio vuoto
  CmdName[Len]=0x00;			// termina la stringa
  return(CmdName);			// termina
}



int ExecCmd(char *CmdLine)
{
  /* Scompone la riga di comando in argv[] e argc, cerca il comando nella
     lista comandi e lo esegue, ritorna 0 normalmente,
     ritorna 1 per chiedere la terminazione del processo. */
    
  

  ShellCmd_t	*Entry;			// puntatore all'entry table
  time_t	Tic,Toc;		// usati per cronometrare l'esecuzione
  int		Argc;			// # elementi nella riga di comando
  int		Len;			// lunghezza riga di comando


    
  // scompone la riga di comando in argv[] e argc
  Len=strlen(CmdLine)-1;		// posizione ultimo char nella stringa
  if(CmdLine[Len] == '\n')		// se termina con ENTER
    CmdLine[Len]=0x00;			// toglilo
  if(!(Argc=ParseLine(CmdLine)))	// se ci sono problemi
    return 0;				// ignora il comando

  // cerca i comandi di terminazione
  if(strcmp(Argv[0],"exit")==0)		// se il comando e' 'exit'
    return 1;				// richiede la terminazione
  if(strcmp(Argv[0],"quit")==0)		// se il comando e' 'quit'
    return 1;				// richiede la terminazione

  // cerca il comando nella lista comandi
#if SHELL_CMD_SECTION
  Entry=&__shell_cmd_start;		// punta alla prima entry
  while(Entry!=&__shell_cmd_end)	// fino a fine tabella
#else
  Entry=CmdList;			// punta alla prima entry
  while(Entry->ActFunc != NULL)		// fino al terminatore
#endif
  {
    if(strcmp(Argv[0],Entry->Cmd)==0)
    {
      time(&Tic);			// istante iniziale
      if(Entry->ActFunc(Argc,Argv))	// esegue il comando
      {
	printf("Usage: %s %s.\n\n",Entry->Cmd,Entry->LongHelp);
      }
      else
      {
        time(&Toc);			// istante finale
	printf("Execution time: %4.2lfs\n",difftime(Toc,Tic));
      }
      return 0;				// termina con successo
    }
    Entry++;				// prossima entry della lista
  }
  printf("Unknown command: %s\n",Argv[0]);
  return 0;				// termina con successo
}

/* $Source$ */

