/*
    Fast buffered I/O access implementation module.
    Copyright (C) 2015  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#include <ftdi.h>				// FTDI lib
#include "io.h"					// slow I/O control

#define __FASTIO_C
#include "fastio.h"				// functions prototypes

/***************************************************************************/
/* define interni alla libreria */

#define FAST_IO_BUFFER_SIZE	128

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* costanti interne alla libreria */

/***************************************************************************/
/* variabili interne alla libreria */

static int		BufferIdx=0;		// indice e occupazione
static unsigned char	FastBuffer[FAST_IO_BUFFER_SIZE];
static unsigned char	IoState=0x00;		// stato delle linee di I/O

/***************************************************************************/
/* funzioni interne alla libreria */

/***************************************************************************/
/* funzioni */

void FAST_SetBusState(unsigned char Data)
{
  /* Imposta lo stato iniziale di tutte le linee di I/O, l'aggiornamento
     dell'I/O e' immediato. */



  IoState=Data;					// modifica lo stato del bus
  IO_DataWrite(Data);				// modifica l'I/O reale
  return;					// termina
}



void FAST_SetBit(int Bit)
{
  /* Alza il bit indicato. */



  IoState|=1<<Bit;				// setta il bit
  return;					// termina
}



void FAST_ClearBit(int Bit)
{
  /* Abbassa il bit indicato. */
  
  
  
  IoState&=~(1<<Bit);				// azzera il bit
  return;					// termina
}



void FAST_RecordState(void)
{
  /* Registra lo stato attuale delle linee di I/O. */



  FastBuffer[BufferIdx]=IoState;		// salva lo stato attuale
  BufferIdx++;					// avanti di 1
  if(BufferIdx >= FAST_IO_BUFFER_SIZE)		// se il buffer e' pieno
    FAST_Flush();				// flushing del buffer
  return;					// termina
}



void FAST_Flush(void)
{
  /* Riproduce quanto registrato sulle linee di I/O reali. */
}

/* $Source$ */

