/*
    HD44780 driver implementation.
    Copyright (C) 2011 2016  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#define __HD44780_C
#include "hd44780.h"				// functions prototypes

#include "io.h"					// hardare control
#define ENABLE_DEBUG_MACROS	1
#include "debug.h"				// debug helper
#include <stdio.h>				// printf
#include <unistd.h>				// usleep

/***************************************************************************/
/* define interni alla libreria */

#define E_LINE		2	// 0x04
#define Rw_LINE		1	// 0x02
#define Dc_LINE		0	// 0x01

#define INPUT		0
#define OUTPUT		1

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* macro interne alla libreria */

#define SetE(v)		IO_BitWrite(E_LINE, v)
#define SetRw(v)	IO_BitWrite(Rw_LINE,v)
#define SetDc(v)	IO_BitWrite(Dc_LINE,v)

/***************************************************************************/
/* costanti interne alla libreria */

/***************************************************************************/
/* variabili interne alla libreria */

/***************************************************************************/
/* funzioni interne alla libreria */

static void SetBus(unsigned char Data);
  /* Mette i 4 bit piu' alti di Data sul bus. */

static unsigned char GetBus(void);
  /* Ritorna nei 4 bit piu' alti lo stato del bus. */

static void SetBusDir(unsigned char Output);
  /* Imposta la direzione del bus. */

static void WriteHalfCommand(unsigned char Data);
  /* Invia la parte alta di Data in un solo ciclo di bus. */
  
static void WaitDisplayDriver(void);
  /* Aspetta che il display driver sia pronto. */

static unsigned char GetStatus(void);
  /* Legge lo stato del display driver. */

static unsigned char GetData(void);
  /* Legge un data byte dal display driver. */

static void PutCommand(unsigned char Cmd);
  /* Invia un byte comando al display driver. */

static void PutData(unsigned char Data);
  /* Invia un byte dati al display driver. */

/***************************************************************************/
/* funzioni */

static void SetBus(unsigned char Data)
{
  /* Mette i 4 bit piu' alti di Data sul bus. */



  FUNCTION_CALL();				// function call trace
  IO_DataWrite((IO_OutDataRead() & 0x0f) | (Data & 0xf0));
  return;					// termina
}



static unsigned char GetBus(void)
{
  /* Ritorna nei 4 bit piu' alti lo stato del bus. */



  FUNCTION_CALL();				// function call trace
  return(IO_DataRead()&0xf0);			// termina
}



static void SetBusDir(unsigned char Output)
{
  /* Imposta la direzione del bus. */



  FUNCTION_CALL();				// function call trace
  if(Output)
    IO_DirMaskWrite(0xff);			// 8 bit in uscita
  else
    IO_DirMaskWrite(0x0f);			// data bus in ingresso
  return;					// termina
}



static void WriteHalfCommand(unsigned char Data)
{
  /* Invia la parte alta di Data in un solo ciclo di bus. */



  FUNCTION_CALL();				// function call trace
  SetRw(0);					// esegue una scrittura
  SetBusDir(OUTPUT);				// bus in uscita
  SetBus(Data);					// 4 bit alti sul bus
  SetE(1);					// alza la linea E
  //usleep(2);					// aspetta
  SetE(0);					// abbassa la linea E
  SetDc(0);					// command mode
  SetRw(1);					// read mode
  SetBusDir(INPUT);				// bus in ingresso
  return;					// termina
}



static void WaitDisplayDriver(void)
{
  /* Aspetta che il display driver sia pronto. */



  FUNCTION_CALL();				// function call trace
  while(GetStatus()&0x80);			// se BUSY=1 aspetta
  return;					// termina
}



static unsigned char GetStatus(void)
{
  /* Legge lo stato del display driver. */



  unsigned char	Data;				// received data byte



  FUNCTION_CALL();				// function call trace
  SetE(1);					// alza la linea E
  //usleep(2);					// aspetta
  Data=GetBus();				// legge il nibble alto
  SetE(0);					// abbassa la linea E
  //usleep(2);					// aspetta
  SetE(1);					// alza la linea E
  //usleep(2);					// aspetta
  Data|=GetBus()>>4;				// legge il nibble basso
  SetE(0);					// abbassa la linea E
  if(Verbosity >= DRIVER_VERBOSITY)
    printf("HD44780 -> 0x%02x\n",(unsigned int)Data);
  return(Data);
}



static unsigned char GetData(void)
{
  /* Legge un data byte dal display driver. */
  
  
  
  unsigned char	Data;				// received data byte



  FUNCTION_CALL();				// function call trace
  SetDc(1);					// data mode
  SetE(1);					// alza la linea E
  //usleep(2);					// aspetta
  Data=GetBus();				// legge il nibble alto
  SetE(0);					// abbassa la linea E
  //usleep(2);					// aspetta
  SetE(1);					// alza la linea E
  //usleep(2);					// aspetta
  Data|=GetBus()>>4;				// legge il nibble basso
  SetE(0);					// abbassa la linea E
  SetDc(0);					// command mode
  if(Verbosity >= DRIVER_VERBOSITY)
    printf("HD44780 -> 0x%02x\n",(unsigned int)Data);
  return(Data);
}



static void PutCommand(unsigned char Data)
{
  /* Invia un byte comando al display driver. */



  FUNCTION_CALL();				// function call trace
  WaitDisplayDriver();				// aspetta il READY del driver
  if(Verbosity >= DRIVER_VERBOSITY)
    printf("HD44780 <- 0x%02x\n",(unsigned int)Data);
  SetRw(0);					// write
  SetBusDir(OUTPUT);				// bus in uscita
  SetBus(Data);					// parte alta sul bus
  SetE(1);					// alza la linea E
  //usleep(2);					// aspetta
  SetE(0);					// abbassa la linea E
  SetBus(Data<<4);				// parte bassa sul bus
  SetE(1);					// alza la linea E
  //usleep(2);					// aspetta
  SetE(0);					// abbassa la linea E
  
  // rimette il bus in posizione di default
  SetDc(0);					// command mode
  SetRw(1);					// read mode
  SetBusDir(INPUT);				// bus in ingresso
  return;					// termina
}



static void PutData(unsigned char Data)
{
  /* Invia un byte dati al display driver. */



  FUNCTION_CALL();				// function call trace
  WaitDisplayDriver();				// aspetta il READY del driver
  if(Verbosity >= DRIVER_VERBOSITY)
    printf("HD44780 <- 0x%02x\n",(unsigned int)Data);
  SetDc(1);					// data mode
  SetRw(0);					// write
  SetBusDir(OUTPUT);				// bus in uscita
  SetBus(Data);					// parte alta sul bus
  SetE(1);					// alza la linea E
  //usleep(2);					// aspetta
  SetE(0);					// abbassa la linea E
  SetBus(Data<<4);				// parte bassa sul bus
  SetE(1);					// alza la linea E
  //usleep(2);					// aspetta
  SetE(0);					// abbassa la linea E
  
  // rimette il bus in posizione di default
  SetDc(0);					// command mode
  SetRw(1);					// read mode
  SetBusDir(INPUT);				// bus in ingresso
  return;					// termina
}



int HD44780_InitBus(void *Chipset,int argc,char **argv)
{
  /* Imposta la direzione e lo stato iniziale delle linee del bus. */



  int	Lines=1;				// numero di linee



  FUNCTION_CALL();				// function call trace
  
  // controlla il numero di parametri passati
  if((argc < 3) || (sscanf(argv[2],"%i",&Lines) != 1))
  {
    printf("Usage:\n%s %s <num_of_lines>\n"
		"num_of_lines can be 1 or 2.",argv[0],argv[1]);
    return 1;					// termina con fallimento
  }

//  // imposta la velocita' del bus
//  IO_SetBaudRate(1000000);			// bus speed

  // imposta lo stato iniziale del bus
  SetE(0);					// abbassa E
  SetRw(1);					// read
  SetDc(0);					// status
  SetBusDir(INPUT);				// bus in ingresso
  usleep(400);					// aspetta

  // inizializza il display driver
  WriteHalfCommand(0x20+0x10);			// imposta il bus a 8 bit
  usleep(400);					// aspetta
  WriteHalfCommand(0x20+0x10);			// imposta il bus a 8 bit
  usleep(41);					// aspetta
  WriteHalfCommand(0x20+0x10);			// imposta il bus a 8 bit
  usleep(2);					// aspetta
  WriteHalfCommand(0x20+0x00);			// imposta il bus a 4 bit
  usleep(2);					// aspetta

  if(Lines == 1)
    PutCommand(0x28);				// 4bit, 1 line, font 5x8
  else
    PutCommand(0x28);				// 4bit, 2 lines, font 5x8
  PutCommand(0x0f);				// display, cursor, blink
  PutCommand(0x01);				// clear display
  return 0;					// termina con successo
}



int HD44780_CmdWrite(const unsigned char CmdData[],int Len)
{
  /* Invia un array di byte comando al display driver. */



  FUNCTION_CALL();				// function call trace
  while(Len--)					// fi che ce n'e'
    PutCommand(*CmdData++);			// manda un comando
  return 0;					// termina con successo
}



int HD44780_TextWrite(unsigned long int Addr,const char *Text,int Len)
{
  /* Stampa una stringa di testo alla posizione attuale del cursore. */



  FUNCTION_CALL();				// function call trace
  while(Len--)					// fin che ce n'e'
    PutData((unsigned char)*Text++);		// stampa un char
  return 0;					// termina con successo
}

/* $Source$ */

