/*
    I/O level handling functions.
    Copyright (C) 2010 2013 2015 2016  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */

#ifndef __IO_H
#define __IO_H	1


/***************************************************************************/
/* define pubblici */

  // pin direction
#define INPUT			0		// pin in ingresso
#define OUTPUT			1		// pin in uscita

  // livelli di verbosita'
#define DRIVER_VERBOSITY	3		// livello driver
#define BUS_VERBOSITY		4		// livello bus
#define IO_VERBOSITY		5		// livello pin I/O


#define IO_BUS_WIDTH		8		// larghezza del bus in bit

/***************************************************************************/
/* macro pubbliche */

/***************************************************************************/
/* strutture pubbliche */

typedef unsigned char		BusData_t;

/***************************************************************************/
/* costanti pubbliche */

/***************************************************************************/
/* variabili pubbliche */

#ifdef __IO_C

unsigned int			Verbosity=0;	// livello di verbosita'

#else

extern unsigned int		Verbosity;	// livello di verbosita'

#endif

/***************************************************************************/
/* funzioni pubbliche */

int IO_Open(int Vendor,int Product,int HardIfaces);
  /* Inizializza il controllo diretto delle linee di I/O e imposta lo stato
     iniziale, ritorna 0 in caso di successo. */
  /**< \brief Opens and initilizes the hardware level.
       \param Vendor
       \param Product
       \param HardIfaces
       \return returns 0 on success.

       This function initializes the hardware level, must be called before
       any other function.
   */

int IO_Close(void);
  /* Rilascia le risorse allocate e chiude l'interfaccia con l'hardware. */


int IO_SetBaudRate(int Baud);
  /* Imposta il baudrate di aggiornamento dell'I/O. */


BusData_t IO_DirMaskRead(void);
  /* Restituisce la maschera direzioni corrente. */

int IO_DirMaskWrite(BusData_t DirMask);
  /* Imposta la direzione dei pin di I/O, ritorna 0 in caso di sucesso. */

int IO_DirMaskSet(BusData_t SetMask);
  /* Setta i bit impostati nella maschera, ritorna 0 in caso di sucesso. */

int IO_DirMaskReset(BusData_t ResetMask);
  /* Resetta i bit impostati nella maschera, ritorna 0 in caso di sucesso. */

int IO_DirMaskToggle(BusData_t ToggleMask);
  /* Inverte i bit impostati nella maschera, ritorna 0 in caso di sucesso. */


BusData_t IO_DirMaskBitRead(unsigned short int Bit);
  /* Restituisce la direzione del pin indicato. */

int IO_DirMaskBitWrite(unsigned short int Bit,BusData_t NewDir);
  /* Imposta la direzione dei pin indicato, ritorna 0 in caso di sucesso. */

int IO_DirMaskBitSet(unsigned short int Bit);
  /* Setta la direzione dei pin indicato, ritorna 0 in caso di sucesso. */

int IO_DirMaskBitReset(unsigned short int Bit);
  /* Resetta la direzione dei pin indicato, ritorna 0 in caso di sucesso. */

int IO_DirMaskBitToggle(unsigned short int Bit);
  /* Inverte la direzione dei pin indicato, ritorna 0 in caso di sucesso. */


BusData_t IO_OutDataRead(void);
  /* Legge lo stato di tutti i pin in uscita. */

BusData_t IO_DataRead(void);
  /* Legge lo stato di tutti i pin in ingresso. */

void IO_DataWrite(BusData_t DataMask);
  /* Imposta lo stato di tutti i pin in uscita. */

void IO_Set(BusData_t SetMask);
  /* Setta i bit impostati nella maschera. */

void IO_Reset(BusData_t ResetMask);
  /* Resetta i bit impostati nella maschera. */

void IO_Toggle(BusData_t ToggleMask);
  /* Inverte i bit impostati nella maschera. */


BusData_t IO_BitRead(unsigned short int Bit);
  /* Legge lo stato del pin indicato. */

void IO_BitWrite(unsigned short int Bit,BusData_t NewValue);
  /* Imposta lo stato del pin indicato. */

void IO_BitSet(unsigned short int Bit);
  /* Setta il bit indicato. */

void IO_BitReset(unsigned short int Bit);
  /* Resetta il bit indicato. */

void IO_BitToggle(unsigned short int Bit);
  /* Inverte il bit indicato. */

/***************************************************************************/
#endif

/* $Source$ */

