/*
    Nokia LCD driver, this drivr supports different display controllers:
    S1G15D10, LDS176, PCF8833 re supported.
    Copyright (C) 2011  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#define __NOKIADRV_C
#include "nokiadrv.h"				// functions prototypes

#include "nokiastr.h"				// chipset struct
#include "s1d15G10.h"				// S1D15G10 chipset
#include "lds176.h"				// LDS176 chipset
#include "pcf8833.h"				// PCF8833 chipset

#include <stdio.h>
#include <unistd.h>

/***************************************************************************/
/* define interni alla libreria */

/***************************************************************************/
/* macro interne alla libreria */

#define SetCS(V)	SPI_SetSpare(0,V)
#define SetRES(V)	SPI_SetSpare(1,V)

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* costanti interne alla libreria */

static const unsigned char	Signal[]={MOSI,MISO,CLK,CS,RES,NEN,SEL,DC};
static const ChipSet_t		*SupportedChipSet[]={	S1G15D10_ChipSet,
							LDS176_ChipSet,
							PCF8833_ChipSet};

/***************************************************************************/
/* variabili interne alla libreria */

static const ChipSet_t	*ActiveChipSet=NULL;	// chip-set selezionato

/***************************************************************************/
/* funzioni interne alla libreria */

/***************************************************************************/
/* funzioni */

int NokiaLcd_InitBus(int ChipSet)
{
  /* Inizializza il bus di comunicazione col display controller e
     inizializza il display secondo il ChipSet selezionato. */



  // individua il chipset richiesto
  ActiveChipSet=SupportedChipSet[ChipSet];	// punta al chipset selezionato

  // inizializza il display driver
  switch(ChipSet)
  {
    case S1G15D10:
      return(S1G15D10_InitBus());
  
  
  
  
int S1D15G10_InitBus(void)
{
  /* Imposta lo stato iniziale e la direzione delle linee del bus e
     inizializza il display driver. */



  unsigned char	Data[24];			// command data



  // inizializzail bus
//  if(SPI_InitBus(SPI_CPOL_HI|SPI_SECOND_EDGE|SPI_MSB_FIRST,Signal))
  if(SPI_InitBus(SPI_CPOL_LOW|SPI_FIRST_EDGE|SPI_MSB_FIRST,Signal))
    return 1;					// termina con fallimento
  SetRES(0);					// resetta il driver
  SetCS(1);					// alza CS
  sleep(1);					// pausa
  SetRES(1);					// rilascia il reset

  // display init
  Data[0]=	0x00;
  Data[1]=	0x20;
  Data[2]=	0x0a;
  Data[3]=	0x01;
  S1D15G10_SendCommand(DISCTL,Data);
  Data[0]=	0x01;
  S1D15G10_SendCommand(COMSCN,Data);
  S1D15G10_SendCommand(OSCON);
  S1D15G10_SendCommand(SLPOUT);
  S1D15G10_SetContrast(28,3);			// imposta il contrasto
//  Data[0]=	0x00;
//  S1D15G10_SendCommand(TMPGRD,Data);
  Data[0]=	0x0f;
  S1D15G10_SendCommand(PWRCTR,Data);
  S1D15G10_SendCommand(DISINV);
  S1D15G10_SendCommand(PTLOUT);			// abbandona partial display
  Data[0]=	0x00;
  Data[1]=	0x00;
  Data[2]=	0x01;				// 8 bit grayscale
  S1D15G10_SendCommand(DATCTL,Data);
  Data[0]=	0x00;
  Data[1]=	0x02;
  Data[2]=	0x04;
  Data[3]=	0x06;
  Data[4]=	0x08;
  Data[5]=	0x0a;
  Data[6]=	0x0c;
  Data[7]=	0x0f;
  Data[8]=	0x00;
  Data[9]=	0x02;
  Data[10]=	0x04;
  Data[11]=	0x06;
  Data[12]=	0x08;
  Data[13]=	0x0a;
  Data[14]=	0x0c;
  Data[15]=	0x0f;
  Data[16]=	0x00;
  Data[17]=	0x06;
  Data[18]=	0x09;
  Data[19]=	0x0f;
  S1D15G10_SendCommand(RGBSET8,Data);
  S1D15G10_SendCommand(DISON);
  return 0;					// termina con successo
}

  
  
  
  
  
  
    case LDS176:
      return(LDS176_InitBus());
    case PCF8833:
      return(PCF8833_InitBus());
  }
  return(1);					// termina con fallimento
}



void NokiaLcd_SendCommand(unsigned char Cmd,int DataLen,unsigned char Data[])
{
  /* Invia un byte comando seguito da un array di argomenti a display. */
  
  
  
  int	i;
  
  
  
  printf("%s: 0x%02x",__FUNCTION__,(unsigned int)Cmd);
  SetCS(0);						// abbassa CS
  SPI_DataExchange(9,Cmd);				// invia il comando
  SetCS(1);						// alza CS
  for(i=0;i<DataLen;i++)
  {
    printf(" 0x%02x",(unsigned int)Data[i]);
    SetCS(0);						// abbassa CS
    SPI_DataExchange(9,0x0100 | (unsigned long int)Data[i]);	// dato
    SetCS(1);						// alza CS
  }
  puts("");						// va a capo
  return;						// termina
}



void NokiaLcd_SetWindow(unsigned char x1,unsigned char y1,
			unsigned char x2,unsigned char y2)
{
  /* Imposta l'area accessibile in scrittura sul display. */
  
  
  
  unsigned char	Data[2];			// command data



  Data[0]=	y1;				// start page
  Data[1]=	y2;				// end page
  NokiaLcd_SendCommand(Chip->Page.Op,Chip->Page.Data,Data);
  Data[0]=	x1;				// start column
  Data[1]=	x2;				// end column
  NokiaLcd_SendCommand(Chip->Column.Op,Chip->Column.Data,Data);
  return;					// termina
}



void NokiaLcd_SetContrast(unsigned char Contrast)
{
  /* Imposta il contrasto del display. */
  
  
  
  unsigned char	Data[2];			// command data



  Data[0]=	Contrast;			// V1 value
  Data[1]=	3;				// resistor K
  NokiaLcd_SendCommand(Chip->Contrast.Op,Chip->Contrast.Data,Data);
  return;					// termina
}



void NokiaLcd_CopyFrame(unsigned char *Frame,int FrameLen)
{
  /* Invia un frame buffer al display driver cominciando dall'angolo in alto
     a destra del display. */
}



void NokiaLcd_PrintStr(char *Str)
{
  /* Stampa una stringa a video come se fosse un terminale testuale. */
  
  
  
  NokiaLcd_SetWindow(Chip->Video.x1,Chip->Video.y1,
  		Chip->Video.x2,Chip->Video.y2);		// tutto il display





  NokiaLcd_SendCommand(RAMWR,Frame,FrameLen);	// riempie il display buffer
  return;					// termina
}

/* $Source$ */

