/*
    SPI bus master implementation.
    Copyright (C) 2010 2011 2016  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#define __SPI_C
#include "spi.h"				// functions prototypes

#include "io.h"					// hardware control
#include <stdio.h>				// printf

/***************************************************************************/
/* define interni alla libreria */

/*
#define MOSI_IDX	0	// posizione nell'array di MOSI
#define MISO_IDX	1	// posizione nell'array di MISO
#define CLK_IDX		2	// posizione nell'array di CLK
#define SP1_IDX		3
#define SP2_IDX		4
#define SP3_IDX		5
#define SP4_IDX		6
#define SP5_IDX		7
*/

  // pin mapping
#define MOSI		0
#define MISO		1
#define CLK		2

/***************************************************************************/
/* macro interne alla libreria */

#define SetMosi(v)	IO_BitWrite(MOSI,v)
#define GetMiso()	IO_BitRead(MISO)
#define SetClk(v)	IO_BitWrite(CLK,v)
#define ToggleClk()	IO_BitToggle(CLK)

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* costanti interne alla libreria */

/***************************************************************************/
/* variabili interne alla libreria */

static int		Nbits=8;		// lunghezza singola word
static unsigned char	SpiMode=0x00;		// operating mode

/***************************************************************************/
/* funzioni interne alla libreria */

/***************************************************************************/
/* funzioni */

int SPI_InitBus(int Mode,int BitPerWord)
{
  /* Imposta la direzione e lo stato iniziale delle linee del bus,
     ritorna 0 in caso di successo. */
  
  

  SpiMode=Mode;					// salva la modalita op.
  Nbits=BitPerWord;				// salva la dim. di una word
  IO_SetBaudRate(9600);				// bus speed

  // imposta lo stato iniziale del bus
  SetMosi(0);					// abbassa MOSI
  if(SpiMode & SPI_CPOL_HI)
  {
    SetClk(1);					// il clock riposa alto
  }
  else
  {
    SetClk(0);					// il clock riposa basso
  }

  // imposta tutte le direzioni delle 3 linee gestite
  IO_DirMaskBitSet(MOSI);			// MOSI = out
  IO_DirMaskBitReset(MISO);			// MISO = in
  IO_DirMaskBitSet(CLK);			// CLK = out
  return 0;					// termina con successo
}



unsigned long int SPI_DataExchange(unsigned long int Data)
{
  /* Invia/riceve una word sul bus SPI. */



  int			i;
  unsigned long int	RecvData=0x00;



  if(Verbosity >= BUS_VERBOSITY)
    printf("SPI <- 0x%08lx  Nbits=%i\n",Data,Nbits);
  if(SpiMode & SPI_MSB_FIRST)
  {
    // MSB first mode
    if(SpiMode & SPI_SECOND_EDGE)		// trigger sul secondo fronte
    {
      for(i=0;i<Nbits;i++)
      {
	ToggleClk();				// inverte il clock
	SetMosi(Data & (0x0001<<(Nbits-1)));	// MSB sul bus
	Data<<=1;				// prepara il prossimo bit
	RecvData<<=1;				// fa posto al nuovo bit
	ToggleClk();				// fronte attivo
	RecvData|=(unsigned long int)GetMiso();	// legge MISO
      }
    }
    else
    {
      for(i=0;i<Nbits;i++)
      {
	SetMosi(Data & (0x0001<<(Nbits-1)));	// MSB sul bus
	ToggleClk();				// inverte il clock
	Data<<=1;				// prepara il prossimo bit
	RecvData<<=1;				// fa posto al nuovo bit
	RecvData|=(unsigned long int)GetMiso();	// legge MISO
	ToggleClk();				// fronte attivo
      }
    }
  }
  else
  {
    // LSB first mode
    if(SpiMode & SPI_SECOND_EDGE)		// trigger sul secondo fronte
    {
      for(i=0;i<Nbits;i++)
      {
	ToggleClk();				// inverte il clock
	SetMosi(Data & 0x0001);			// MSB sul bus
	Data>>=1;				// prepara il prossimo bit
	RecvData>>=1;				// fa posto al nuovo bit
	ToggleClk();				// fronte attivo
	RecvData|=((unsigned long int)GetMiso())<<(Nbits-1);	// legge MISO
      }
    }
    else
    {
      for(i=0;i<Nbits;i++)
      {
	SetMosi(Data & 0x0001);			// MSB sul bus
	ToggleClk();				// inverte il clock
	Data>>=1;				// prepara il prossimo bit
	RecvData>>=1;				// fa posto al nuovo bit
	RecvData|=((unsigned long int)GetMiso())<<(Nbits-1);	// legge MISO
	ToggleClk();				// fronte attivo
      }
    }
  }

  // assicura che il clock sia a posto
  if(SpiMode & SPI_CPOL_HI)
  {
    SetClk(1);					// il clock riposa alto
  }
  else
  {
    SetClk(0);					// il clock riposa basso
  }
  if(Verbosity >= BUS_VERBOSITY)
    printf("SPI -> 0x%08lx\n",RecvData);
  return(RecvData);				// termina
}



void SPI_DataWrite(unsigned long int Data)
{
  /* Invia una word sul bus SPI. */
  
  
  
  int			i;



  if(Verbosity >= BUS_VERBOSITY)
    printf("SPI <- 0x%08lx  Nbits=%i\n",Data,Nbits);
  if(SpiMode & SPI_MSB_FIRST)
  {
    // MSB first mode
    if(SpiMode & SPI_SECOND_EDGE)		// trigger sul secondo fronte
    {
      for(i=0;i<Nbits;i++)
      {
	ToggleClk();				// inverte il clock
	SetMosi(Data & (0x0001<<(Nbits-1)));	// MSB sul bus
	Data<<=1;				// prepara il prossimo bit
	ToggleClk();				// fronte attivo
      }
    }
    else
    {
      for(i=0;i<Nbits;i++)
      {
	SetMosi(Data & (0x0001<<(Nbits-1)));	// MSB sul bus
	ToggleClk();				// inverte il clock
	Data<<=1;				// prepara il prossimo bit
	ToggleClk();				// fronte attivo
      }
    }
  }
  else
  {
    // LSB first mode
    if(SpiMode & SPI_SECOND_EDGE)		// trigger sul secondo fronte
    {
      for(i=0;i<Nbits;i++)
      {
	ToggleClk();				// inverte il clock
	SetMosi(Data & 0x0001);			// MSB sul bus
	Data>>=1;				// prepara il prossimo bit
	ToggleClk();				// fronte attivo
      }
    }
    else
    {
      for(i=0;i<Nbits;i++)
      {
	SetMosi(Data & 0x0001);			// MSB sul bus
	ToggleClk();				// inverte il clock
	Data>>=1;				// prepara il prossimo bit
	ToggleClk();				// fronte attivo
      }
    }
  }

  // assicura che il clock sia a posto
  if(SpiMode & SPI_CPOL_HI)
  {
    SetClk(1);					// il clock riposa alto
  }
  else
  {
    SetClk(0);					// il clock riposa basso
  }
  return;					// termina
}

/* $Source$ */

